@@ -5,7 +5,7 @@ use warnings;
 package Sub::HandlesVia::CodeGenerator;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.024';
+our $VERSION   = '0.025';
 
 use Scope::Guard ();
 use Class::Tiny (
@@ -46,8 +46,8 @@ use Class::Tiny (
 		},
 		generator_for_arg => sub {
 			return sub {
-				@_==1 or die;
-				my $n = shift;
+				@_==2 or die;
+				my $n = pop;
 				"\$_[$n]";
 			};
 		},
@@ -58,14 +58,15 @@ use Class::Tiny (
 		},
 		generator_for_currying => sub {
 			return sub {
-				@_==1 or die;
-				my $arr = shift;
+				@_==2 or die;
+				my $arr = pop;
 				"splice(\@_,1,0,$arr);";
 			};
 		},
 		generator_for_usage_string => sub {
 			return sub {
-				@_==2 or die;
+				@_==3 or die;
+				shift;
 				my $method_name = shift;
 				my $guts = shift;
 				"\$instance->$method_name($guts)";
@@ -111,7 +112,7 @@ for my $thing ( @generatable_things ) {
 			return $gen->$coderef( @_ );
 		}
 		
-		return $gen->$generator->( @_ );
+		return $gen->$generator->( $gen, @_ );
 	};
 	no strict 'refs';
 	*$method_name = $method;
@@ -180,6 +181,7 @@ sub generate_coderef_for_handler {
 	
 	my $ec_args = $self->_generate_ec_args_for_handler( $method_name, $handler );
 	
+#	warn "#### $method_name";
 #	warn join("\n", @{$ec_args->{source}});
 #	for my $key (sort keys %{$ec_args->{environment}}) {
 #		warn ">> $key : ".ref($ec_args->{environment}{$key});
@@ -196,26 +198,63 @@ sub generate_coderef_for_handler {
 sub _generate_ec_args_for_handler {
 	my ( $self, $method_name, $handler ) = @_;
 	
+	# Later on, we might need to override the generators for
+	# arg, argc, args, set, etc.
+	#
 	my $guard = $self->_start_overriding_generators;
 	
-	# COPY of $self->env
+	# Make a COPY of $self->env!
+	#
 	my $env = { %{$self->env} };
 	
+	# Preamble code.
+	#
 	my $code = [
 		'sub {',
 		sprintf( 'package %s::__SANDBOX__;', __PACKAGE__ ),
 	];
+
+	# Need to maintain state between following method calls. A proper
+	# object might be nice, but a hashref will do for now.
+	#
+	my $state = {
+		signature_check_needed  => 1,     # hasn't been done yet
+		final_type_check_needed => $handler->is_mutator,
+		getter                  => scalar($self->generate_get),
+		getter_is_lvalue        => $self->get_is_lvalue,
+		template_wrapper        => undef, # nothing yet
+		add_later               => undef, # nothing yet
+		shifted_self            => 0,
+	};
+
+#	use Hash::Util qw( lock_ref_keys );
+#	lock_ref_keys( $state );
 	
-	my $state = {};
-	
+	my @args = (
+		$method_name,  # Intended name for the coderef being generated
+		$handler,      # Info about the functionality being delegated
+		$env,          # Variables which need to be closed over
+		$code,         # Lines of code in the method
+		$state,        # Shared state while building method. (Minimal!)
+	);
 	$self
-		->__process_sigcheck( $method_name, $handler, $env, $code, $state )
-		->__process_currying( $method_name, $handler, $env, $code, $state )
-		->__process_handler_template( $method_name, $handler, $env, $code, $state )
-		->__process_chaining( $method_name, $handler, $env, $code, $state );
+		->_handle_sigcheck( @args )               # check method sigs
+		->_handle_shiftself( @args )              # $self = shift
+		->_handle_currying( @args )               # push curried values to @_
+		->_handle_additional_validation( @args )  # additional type checks
+		->_handle_getter_code( @args )            # optimize calling getter
+		->_handle_setter_code( @args )            # make calling setter safer
+		->_handle_template( @args )               # perform code substitutes
+		->_handle_chaining( @args );              # return $self if requested
 	
+	# Postamble code. Can't really do much here because the template
+	# might want to be able to return something.
+	#
 	push @$code, "}";
 	
+	# Allow the handler to inject variables into the environment.
+	# Rarely needed.
+	#
 	$handler->_tweak_env( $env );
 	
 	return {
@@ -229,36 +268,119 @@ sub _generate_ec_args_for_handler {
 	};
 }
 
-# Insert code into method for signature validation.
-#
-sub __process_sigcheck {
+sub _handle_shiftself {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
-	$state->{signature_check_needed} = 1;
 
+	# Handlers which use @ARG will benefit from shifting $self
+	# off @_, but for other handlers, this will just slow compilation
+	# down (but not much).
+	#
+	return $self
+		unless $handler->curried || $handler->prefer_shift_self;
+
+	# Shift off the invocant.
+	#
+	push @$code, 'my $shv_self=shift;';
+	
+	$self->_add_generator_override(
+	
+		# Override $ARG[$n] because the array has been reindexed.
+		#
+		arg  => sub { my ($gen, $n) = @_; $gen->generate_arg( $n - 1 ) },
+		
+		# Overrride @ARG to point to the whole array. This is the
+		# real speed-up!
+		#
+		args => sub { '@_' },
+		
+		# Override #ARG to no longer subtract 1.
+		#
+		argc => sub { 'scalar(@_)' },
+		
+		# $SELF is now '$shv_self'.
+		#
+		self => sub { '$shv_self' },
+		
+		# The default currying callback will splice the list into
+		# @_ at index 1. Instead unshift the list at the start of @_.
+		#
+		currying => sub {
+			my ($gen, $list) = @_;
+			"CORE::unshift(\@_, $list);";
+		},
+	);
+	
+	# Getter was cached in $state and needs update.
+	#
+	$state->{getter} = $self->generate_get;
+	$state->{shifted_self} = 1;
+	
+	return $self;
+}
+
+sub _handle_sigcheck {
+	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
+
+	# If there's a proper signature for the method...
+	#
 	if ( @{ $handler->signature || [] } ) {
+		
+		# Generate code using Type::Params to check the signature.
+		# We also need to close over the signature.
+		#
 		require Type::Params;
 		unshift @$code, 'my $__sigcheck;';
 		$env->{'@__sig'} = $handler->signature;
-		push @$code, '$__sigcheck||=Type::Params::compile(1, @__sig);@_=&$__sigcheck;';
+		if ( $state->{shifted_self} ) {
+			push @$code, '$__sigcheck||=Type::Params::compile(@__sig);@_=&$__sigcheck;';
+		}
+		else {
+			push @$code, '$__sigcheck||=Type::Params::compile(1, @__sig);@_=&$__sigcheck;';
+		}
+		
+		# As we've now inserted a signature check, we can stop worrying
+		# about signature checks.
+		#
 		$state->{signature_check_needed} = 0;
 	}
+	# There is no proper signature, but there's still check the
+	# arity of the method.
+	#
 	else {
-		my $min_args = $handler->has_min_args ? $handler->min_args : 0;
+		# What is the arity?
+		#
+		my $min_args = $handler->min_args || 0;
 		my $max_args = $handler->max_args;
+		
+		my $plus = 1;
+		if ( $state->{shifted_self} ) {
+			$plus = 0;
+		}
+		
+		# What usage message do we want to print if wrong arity?
+		#
 		my $usg = sprintf(
 			'do { require Carp; Carp::croak("Wrong number of parameters; usage: ".%s) }',
 			B::perlstring( $self->generate_usage_string( $method_name, $handler->usage ) ),
 		);
 		
+		# Insert the check into the code.
+		#
 		if (defined $min_args and defined $max_args and $min_args==$max_args) {
-			push @$code, sprintf('@_==%d or %s;', $min_args + 1, $usg);
+			push @$code, sprintf('@_==%d or %s;', $min_args + $plus, $usg);
 		}
 		elsif (defined $min_args and defined $max_args) {
-			push @$code, sprintf('(@_ >= %d and @_ <= %d) or %s;', $min_args + 1, $max_args + 1, $usg);
+			push @$code, sprintf('(@_ >= %d and @_ <= %d) or %s;', $min_args + $plus, $max_args + $plus, $usg);
 		}
-		elsif (defined $min_args) {
-			push @$code, sprintf('@_ >= %d or %s;', $min_args + 1, $usg);
+		elsif (defined $min_args and $min_args > 0) {
+			push @$code, sprintf('@_ >= %d or %s;', $min_args + $plus, $usg);
 		}
+		
+		# We are still lacking a proper signature check though, so note
+		# that in the state. The information can be used by
+		# additional_validation coderefs.
+		#
+		$state->{signature_check_needed} = 1;
 	}
 	
 	return $self;
@@ -266,14 +388,25 @@ sub __process_sigcheck {
 
 # Insert code into method for currying.
 #
-sub __process_currying {
+sub _handle_currying {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
 	
 	if ( my $curried = $handler->curried ) {
+		
+		# If the curried values are non-simple, we close over an array
+		# called @curry.
+		#
 		if ( grep ref, @$curried ) {
-			$env->{'@curry'} = $curried;
+			
+			# Note that generate_currying will generate code that unshifts whatever
+			# parameters it is given onto @_.
 			push @$code, $self->generate_currying('@curry');
-		} else {
+			$env->{'@curry'} = $curried;
+		}
+		# If it's just strings, numbers, and undef, it should be pretty
+		# trivial to hard-code the values into the generated Perl string.
+		#
+		else {
 			require B;
 			my $values = join(
 				',',
@@ -286,88 +419,85 @@ sub __process_currying {
 	return $self;
 }
 
-sub __process_handler_template {
+sub _handle_additional_validation {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
 	
-	# If the handler is a mutator, then a type check will be
-	# needed later when setting the new value.
+	# If the handler specifies no validation needed, or the attribute
+	# simply has no type check, we don't need to check the type of the
+	# final attribute value.
 	#
-	$state->{return_type_check_needed} = $handler->is_mutator;
 	if ( $handler->no_validation_needed or not $self->isa ) {
-		$state->{return_type_check_needed} = 0;
+		$state->{final_type_check_needed} = 0;
 	}
 	
-	$self
-		->__process_additional_validation( $method_name, $handler, $env, $code, $state )
-		->__process_getter_code( $method_name, $handler, $env, $code, $state )
-		->__process_setter_code( $method_name, $handler, $env, $code, $state );
-	
-	my $template = $self->__choose_template( $method_name, $handler, $env, $code, $state );
-	
-	$template =~ s/\$SELF/$self->generate_self()/eg;
-	$template =~ s/\$SLOT/$self->generate_slot()/eg;
-	$template =~ s/\$GET/$state->{getter}/g;
-	$template =~ s/\$ARG\[([0-9]+)\]/$self->generate_arg($1)/eg;
-	$template =~ s/\$ARG/$self->generate_arg(1)/eg;
-	$template =~ s/\#ARG/$self->generate_argc()/eg;
-	$template =~ s/\@ARG/$self->generate_args()/eg;
-	$template =~ s/«(.+?)»/$self->generate_set($1)/eg;
-	$template =~ s/\$DEFAULT/$self->generate_default($self, $handler)/eg;
-	
-	# Apply wrapper
-	$template = sprintf( $state->{template_wrapper}, $template )
-		if $state->{template_wrapper};
-	
-	# If validation needs to be added late...
-	$template =~ s/\"?____VALIDATION_HERE____\"?/$state->{add_later}/
-		if defined $state->{add_later};
-	
-	push @$code, $template;
-	
-	return $self;
-}
-
-sub __process_additional_validation {
-	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
-	
 	# The handler can define some additional validation to be performed
 	# on arguments either now or later, such that if this additional
 	# validation is performed, the type check we were planning later
 	# will be known to be unnecessary.
 	#
-	my $additional_validation_opts;
-	if ( $state->{return_type_check_needed} and defined $handler->additional_validation ) {
-		$additional_validation_opts = $handler->_real_additional_validation->(
-			$handler,
-			!$state->{signature_check_needed},
-			$self,
-		) || {};
+	# An example for this is that is the attribute value is already an
+	# arrayref of numbers, and we're pushing a new value onto it, by checking
+	# up front that the INCOMING value is a number, it becomes unnecessary
+	# to check the whole arrayref contains numbers after the push.
+	#
+	# Not all handlers define an additional_validation coderef to do
+	# this, because in many cases it doesn't make sense to.
+	#
+	# Also if we've already decided a final type check isn't needed, we
+	# can skip this step.
+	#
+	if ( $state->{final_type_check_needed}
+	and  defined $handler->additional_validation ) {
 		
-		$self->_add_generator_override( %$additional_validation_opts );
+		my $real_av_method = $handler->_real_additional_validation;
 		
-		if ($additional_validation_opts->{add_later}) {
-			$state->{add_later} = $additional_validation_opts->{code};
-			$env->{$_} = $additional_validation_opts->{env}{$_}
-				for keys %{ $additional_validation_opts->{env} };
-			$state->{return_type_check_needed} = 0;
-		}
-		elsif ($additional_validation_opts->{code}) {
-			push @$code, $additional_validation_opts->{code};
-			$env->{$_} = $additional_validation_opts->{env}{$_}
-				for keys %{ $additional_validation_opts->{env} };
-			$state->{return_type_check_needed} = 0;
+		# The additional_validation coderef is called as a method and takes
+		# two additional parameters:
+		#
+		my $opt = $handler->$real_av_method(
+			!$state->{signature_check_needed},  # $sig_was_checked
+			$self,                              # $gen
+		);
+		$opt ||= {}; # can return undef
+		
+		# The additional_validation coderef will often generate code which
+		# coerces incoming data, thus moving it from @_ to some other array.
+		# This means that the generators for @ARG, $ARG, etc will need to
+		# need to be overridden to point to the new array.
+		#
+		$self->_add_generator_override( %$opt );
+		
+		# The additional_validation coderef may supply extra variables
+		# to close over.
+		#
+		$env->{$_} = $opt->{env}{$_}
+			for keys %{ $opt->{env} || {} };
+		
+		# The additional_validation coderef will normally generate
+		# new code.
+		#
+		if ( defined $opt->{code} ) {
+			
+			# Code can be inserted into the generated method straight away,
+			# or may need to be inserted in a special placeholder position
+			# later.
+			#
+			$opt->{add_later}
+				? ( $state->{add_later} = $opt->{code} )
+				: push( @$code, $opt->{code} );
+			
+			# It is assumed that a final type check is no longer needed.
+			#
+			$state->{final_type_check_needed} = 0;
 		}
 	}
 	
 	return $self;
 }
 
-sub __process_getter_code {
+sub _handle_getter_code {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
 	
-	$state->{getter} = $self->generate_get();
-	$state->{getter_is_lvalue} = $self->get_is_lvalue;
-	
 	# If there's a complicated way to fetch the attribute value (perhaps
 	# involving a lazy builder)...
 	#
@@ -385,7 +515,7 @@ sub __process_getter_code {
 		
 		# Alternatively, unless the handler doesn't want us to, or the template
 		# doesn't want to get the attribute value anyway, then we'll do something
-		# similar.
+		# similar. Here it can't be used as an lvalue though.
 		#
 		elsif ( $handler->allow_getter_shortcuts
 		and $handler->template.($handler->lvalue_template||'') =~ /\$GET/ ) {
@@ -398,16 +528,16 @@ sub __process_getter_code {
 	return $self;
 }
 
-# Possibly add a type check to the setter.
-#
-sub __process_setter_code {
+sub _handle_setter_code {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
 	
 	# If a type check is needed, but the setter doesn't do type checks,
 	# then override the setter. Now the setter does the type check, so
-	# we no longer need to.
+	# we no longer need to worry about it.
 	#
-	if ( $state->{return_type_check_needed} and not $self->set_checks_isa ) {
+	# XXX: I don't think any of the tests currently exercise this.
+	#
+	if ( $state->{final_type_check_needed} and not $self->set_checks_isa ) {
 		$self->_add_generator_override( set => sub {
 			my ( $me, $value_code ) = @_;
 			$me->generate_set( sprintf(
@@ -417,14 +547,25 @@ sub __process_setter_code {
 			) );
 		} );
 		$env->{'$shv_final_type'} = \( $self->isa );
+		
+		# In this case we can no longer use the getter as an lvalue, if we
+		# ever could.
+		#
 		$state->{getter_is_lvalue} = 0;
-		$state->{return_type_check_needed} = 0;
+		
+		# Stop worrying about the final type check. The setter does that now.
+		#
+		$state->{final_type_check_needed} = 0;
 	}
+	
+	return $self;
 }
 
-sub __choose_template {
+sub _handle_template {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
-
+	
+	my $template;
+	
 	# If the getter is an lvalue, the handler has a special template
 	# for lvalues, we haven't been told to set strictly, and we have taken
 	# care of any type checks, then use the special lvalue template.
@@ -432,16 +573,46 @@ sub __choose_template {
 	if ( $state->{getter_is_lvalue}
 	and  $handler->lvalue_template
 	and  !$self->set_strictly
-	and  !$state->{return_type_check_needed} ) {
-		return $handler->lvalue_template;
+	and  !$state->{final_type_check_needed} ) {
+		$template = $handler->lvalue_template;
+	}
+	else {
+		$template = $handler->template;
 	}
 	
-	return $handler->template;
+	# Perform substitutions of special codes in the template string.
+	#
+	$template =~ s/\$SLOT/$self->generate_slot()/eg;
+	$template =~ s/\$GET/$state->{getter}/g;
+	$template =~ s/\$ARG\[([0-9]+)\]/$self->generate_arg($1)/eg;
+	$template =~ s/\$ARG/$self->generate_arg(1)/eg;
+	$template =~ s/\#ARG/$self->generate_argc()/eg;
+	$template =~ s/\@ARG/$self->generate_args()/eg;
+	$template =~ s/«(.+?)»/$self->generate_set($1)/eg;
+	$template =~ s/\$DEFAULT/$self->generate_default($handler)/eg;
+	$template =~ s/\$SELF/$self->generate_self()/eg;
+	
+	# Apply wrapper (if any). This wrapper is given
+	# by _handle_getter_code (sometimes).
+	#
+	$template = sprintf( $state->{template_wrapper}, $template )
+		if $state->{template_wrapper};
+	
+	# If validation needs to be added late...
+	#
+	$template =~ s/\"?____VALIDATION_HERE____\"?/$state->{add_later}/
+		if defined $state->{add_later};
+	
+	push @$code, $template;
+	
+	return $self;
 }
 
-sub __process_chaining {
+sub _handle_chaining {
 	my ( $self, $method_name, $handler, $env, $code, $state ) = @_;
 	
+	# Will just insert a string like ';$_[0]' at the end
+	#
 	push @$code, ';' . $self->generate_self,
 		if $handler->is_chainable;
 	
@@ -517,12 +688,15 @@ will install the method. Note that it isn't passed the package to install
 into (which can be found in C<target>), so that would need to be closed
 over.
 
-=head2 C<generator_for_slot> B<< CodeRef >>
+=head2 C<generator_for_self> B<< CodeRef >>
 
 A coderef which if called, generates a string like C<< '$_[0]' >>.
 
 Has a sensible default.
 
+All the C<generator_for_XXX> methods are called as methods, so have
+the code generator object as an invocant.
+
 =head2 C<generator_for_slot> B<< CodeRef >>
 
 A coderef which if called, generates a string like C<< '$_[0]{attrname}' >>.
@@ -541,6 +715,9 @@ C<< "\$_[0]->_set_attrname( $parameter )" >>.
 A coderef which if called with a parameter, generates a string like
 C<< 'undef' >> or C<< 'q[]' >> or C<< '{}' >>.
 
+The parameter is a handler object, which offers a C<default_for_reset>
+attribute which might be able to provide a useful fallback.
+
 =head2 C<generator_for_args> B<< CodeRef >>
 
 A coderef which if called, generates a string like C<< '@_[1..$#_]' >>.
@@ -572,7 +749,8 @@ Has a sensible default.
 The default is this coderef:
 
   sub {
-    @_==2 or die;
+    @_==3 or die;
+    shift;
     my $method_name = shift;
     my $guts = shift;
     return "\$instance->$method_name($guts)";
