@@ -5,7 +5,7 @@ use warnings;
 package Sub::HandlesVia::HandlerLibrary::Array;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.024';
+our $VERSION   = '0.025';
 
 use Sub::HandlesVia::HandlerLibrary;
 our @ISA = 'Sub::HandlesVia::HandlerLibrary';
@@ -52,23 +52,30 @@ my $additional_validation_for_push_and_unshift = sub {
 	
 	if ($ti and $ti->{trust_mutated} eq 'maybe') {
 		my $coercion = ($gen->coerce && $ti->{value_type}->has_coercion);
-		my $code = $coercion
-			? sprintf(
+		if ( $coercion ) {
+			my $code = sprintf(
 				'my @shv_values = map $shv_type_for_values->assert_coerce($_), %s;',
 				$gen->generate_args,
-			)
-			: sprintf(
-				'my @shv_values = %s; for my $shv_value (@shv_values) { %s }',
+			);
+			return {
+				code      => $code,
+				env       => { '$shv_type_for_values' => \$ti->{value_type} },
+				arg       => sub { CORE::shift; "\$shv_values[($_[0])-1]" },
+				args      => sub { CORE::shift; '@shv_values' },
+				argc      => sub { CORE::shift; 'scalar(@shv_values)' },
+			};
+		}
+		else {
+			my $code = sprintf(
+				'for my $shv_value (%s) { %s }',
 				$gen->generate_args,
 				$ti->{value_type}->inline_assert('$shv_value', '$shv_type_for_values'),
 			);
-		return {
-			code      => $code,
-			env       => { '$shv_type_for_values' => \$ti->{value_type} },
-			arg       => sub { CORE::shift; "\$shv_values[($_[0])-1]" },
-			args      => sub { CORE::shift; '@shv_values' },
-			argc      => sub { CORE::shift; 'scalar(@shv_values)' },
-		};
+			return {
+				code      => $code,
+				env       => { '$shv_type_for_values' => \$ti->{value_type} },
+			};
+		}
 	}
 	return;
 };
@@ -240,6 +247,7 @@ sub push {
 		usage     => '@values',
 		template  => 'my @shv_tmp = @{$GET}; my $shv_return = push(@shv_tmp, @ARG); «\\@shv_tmp»; $shv_return',
 		lvalue_template => 'push(@{$GET}, @ARG)',
+		prefer_shift_self => 1,
 		additional_validation => $additional_validation_for_push_and_unshift,
 		documentation => 'Adds elements to the end of the array.',
 		_examples => sub {
@@ -279,6 +287,7 @@ sub unshift {
 		usage     => '@values',
 		template  => 'my @shv_tmp = @{$GET}; my $shv_return = unshift(@shv_tmp, @ARG); «\\@shv_tmp»; $shv_return',
 		lvalue_template => 'unshift(@{$GET}, @ARG)',
+		prefer_shift_self => 1,
 		additional_validation => $additional_validation_for_push_and_unshift,
 		documentation => 'Adds an element to the start of the array.',
 		_examples => sub {
