@@ -5,7 +5,7 @@ use warnings;
 package Sub::HandlesVia::Toolkit::Mouse;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.024';
+our $VERSION   = '0.025';
 
 use Sub::HandlesVia::Toolkit;
 our @ISA = 'Sub::HandlesVia::Toolkit';
@@ -55,27 +55,42 @@ sub code_generator_for_attribute {
 	if (!$spec->{lazy} and !$spec->{traits} and !$spec->{auto_deref}) {
 		require B;
 		my $slot = B::perlstring($attrname);
-		$get = sub { "\$_[0]{$slot}" };
+		$get = sub {
+			my $self = shift->generate_self;
+			"$self\->{$slot}";
+		};
 		++$get_is_lvalue;
 	}
 	elsif ($attr->has_read_method) {
 		my $read_method = $attr->reader || $attr->accessor;
-		$get = sub { "scalar(\$_[0]->$read_method)" };
+		$get = sub {
+			my $self = shift->generate_self;
+			"scalar($self\->$read_method)";
+		};
 	}
 	else {
 		my $read_method = $attr->get_read_method_ref;
 		$captures->{'$shv_read_method'} = \$read_method;
-		$get = sub { 'scalar($_[0]->$shv_read_method)' };
+		$get = sub {
+			my $self = shift->generate_self;
+			"scalar($self\->\$shv_read_method)";
+		};
 	}
 	if ($attr->has_write_method) {
 		my $write_method = $attr->writer || $attr->accessor;
-		$set = sub { my $val = shift; "\$_[0]->$write_method\($val)" };
+		$set = sub {
+			my ($gen, $val) = @_;
+			$gen->generate_self . "->$write_method\($val)"
+		};
 		++$set_checks_isa;
 	}
 	else {
 		my $write_method = $attr->get_write_method_ref;
 		$captures->{'$shv_write_method'} = \$write_method;
-		$set = sub { my $val = shift; '$_[0]->$shv_write_method('.$val.')' };
+		$set = sub {
+			my ($gen, $val) = @_;
+			$gen->generate_self . '->$shv_write_method('.$val.')';
+		};
 		++$set_checks_isa;
 	}
 
@@ -100,7 +115,7 @@ sub code_generator_for_attribute {
 		env                   => $captures,
 		isa                   => Types::TypeTiny::to_TypeTiny($attr->type_constraint),
 		coerce                => !!$spec->{coerce},
-		generator_for_slot    => sub { '$_[0]{'.B::perlstring($attrname).'}' }, # icky
+		generator_for_slot    => sub { shift->generate_self.'->{'.B::perlstring($attrname).'}' }, # icky
 		generator_for_get     => $get,
 		generator_for_set     => $set,
 		get_is_lvalue         => $get_is_lvalue,
@@ -115,14 +130,14 @@ sub code_generator_for_attribute {
 			elsif ( $default->[0] eq 'builder' ) {
 				return sprintf(
 					'(%s)->%s',
-					$gen->generator_for_self->(),
+					$gen->generate_self,
 					$default->[1],
 				);
 			}
 			elsif ( $default->[0] eq 'default' and ref $default->[1] eq 'CODE' ) {
 				return sprintf(
 					'(%s)->$shv_default_for_reset',
-					$gen->generator_for_self->(),
+					$gen->generate_self,
 				);
 			}
 			elsif ( $default->[0] eq 'default' and !defined $default->[1] ) {
@@ -140,7 +155,7 @@ sub code_generator_for_attribute {
 package Sub::HandlesVia::Toolkit::Mouse::PackageTrait;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.024';
+our $VERSION   = '0.025';
 
 use Role::Tiny;
 
@@ -178,7 +193,7 @@ around add_attribute => sub {
 package Sub::HandlesVia::Toolkit::Mouse::RoleTrait;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.024';
+our $VERSION   = '0.025';
 
 use Role::Tiny;
 
